#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.settings.Settings import Settings

import libsbml
from libsedml import XMLNode
from six.moves import reload_module
reload_module(libsbml)


class NewXML(object):

	def __init__(self, document):
		self.__document = document
		self.__xmlNode = None

	def readSedml(self, xml_node, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		self.__xmlNode = xml_node

	def writeSedml(self, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		return self.__xmlNode

	def setFromString(self, xml_string):
		self.__xmlNode = XMLNode.convertStringToXMLNode(xml_string)

	def getAsString(self):
		return XMLNode.convertXMLNodeToString(self.__xmlNode)

	def getXMLNode(self):
		return self.__xmlNode