#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.LibSigNetSimException import LibSigNetSimException


class SedmlException(LibSigNetSimException):
	pass

class SedmlMathException(SedmlException):
	pass

class SedmlModelLanguageNotSupported(SedmlException):
	pass

class SedmlModelNotFound(SedmlException):
	pass

class SedmlFileNotFound(SedmlException):
	pass

class SedmlUnknownURI(SedmlException):
	pass

class SedmlUnknownXPATH(SedmlException):
	pass

class SedmlNotImplemented(SedmlException):
	pass

# One step simulations cannot be executed as a single task, they must be part of a repeated task
# At least, that's what I understand
class SedmlOneStepTaskException(SedmlException):
	pass

# Mixed subtasks are not implemented yet
class SedmlMixedSubtasks(SedmlNotImplemented):
	pass

class SedmlMultipleModels(SedmlNotImplemented):
	pass

class SedmlModelObjectNotFound(SedmlException):
	pass